# Generated by roxygen2: do not edit by hand

export(cancel_slurm)
export(cleanup_files)
export(get_job_status)
export(get_slurm_out)
export(local_slurm_array)
export(print_job_status)
export(slurm_apply)
export(slurm_call)
export(slurm_job)
export(slurm_map)
importFrom(parallel,mclapply)
importFrom(utils,capture.output)
importFrom(utils,read.table)
