% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_job_status.R, R/rslurm-deprecated.R
\name{print_job_status}
\alias{print_job_status}
\alias{rslurm-deprecated}
\title{Deprecated functions in package \pkg{rslurm}.}
\usage{
print_job_status(slr_job)
}
\description{
The functions listed below are deprecated and will be defunct in
  the near future. When possible, alternative functions with similar
  functionality are also mentioned. Help pages for deprecated functions are
  available at \code{help("-deprecated")}.
}
\section{\code{print_job_status}}{

For \code{print_job_status}, use \code{\link{get_job_status}}.
}

\keyword{internal}
